using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace StripTeamFoundationServerInformation
{
	class MainClass
	{
		public static void Main (string[] args)
		{
			Console.WriteLine ("StripTeamFoundationServerInformation tool");
			Console.WriteLine ("(C) sones GmbH 2010 (Developer: Daniel Kirstenpfad)");
			Console.WriteLine ("    This small tool removes all team foundation server references from solution and project files.");
			Console.WriteLine();
			
			if (args.Length == 0)
			{
				Console.WriteLine("Please give a directory where to strip everything (recursively)");
				Console.WriteLine();
				Console.WriteLine("");
				
				return;
			}
			
        TraverseTree(args[0]);

        Console.WriteLine("Press enter to finish...");
        Console.ReadLine();
		}
		
		#region Strip a solution file
		public static void StripSLN(string file_url)
		{
			StringBuilder Output = new StringBuilder();
			Boolean dontusethis = false;
			
			Console.WriteLine("Stripping {0}",file_url);
			
			using (StreamReader sr = new StreamReader(file_url)) 
            {
                while (sr.Peek() >= 0)
                {
					String line = sr.ReadLine();
					
                    if (line.Contains("GlobalSection(TeamFoundationVersionControl)"))
						dontusethis = true;
					
					if (!dontusethis)
						Output.AppendLine(line);
					else
					{
						if (line.Contains("EndGlobalSection"))
							dontusethis = false;
					}
                }
            }
			
			using (StreamWriter sw = new StreamWriter(file_url)) 
            {
				sw.Write(Output.ToString());
            }
		}
		#endregion
		
		#region Strip a project file
		public static void StripCSPROJ(string file_url)
		{
			StringBuilder Output = new StringBuilder();
			
			Console.WriteLine("Stripping {0}",file_url);
			
			using (StreamReader sr = new StreamReader(file_url)) 
            {
                while (sr.Peek() >= 0)
                {
					String line = sr.ReadLine();
					
                    if (line.Contains("SccProjectName"))
						continue;
                    if (line.Contains("SccLocalPath"))
						continue;
	                if (line.Contains("SccAuxPath"))
						continue;
					if (line.Contains("SccProvider"))
						continue;

					Output.AppendLine(line);
                }
            }
			
			using (StreamWriter sw = new StreamWriter(file_url)) 
            {
				sw.Write(Output.ToString());
            }
		}
		#endregion
		
		#region Traverse Directory tree
		public static void TraverseTree(string root)
		{
		// Data structure to hold names of subfolders to be
		// examined for files.
		Stack<string> dirs = new Stack<string>(20);
		
		if (!System.IO.Directory.Exists(root))
		{
		    throw new ArgumentException();
		}
		dirs.Push(root);
		
		while (dirs.Count > 0)
		{
		    string currentDir = dirs.Pop();
		    string[] subDirs;
		    try
		    {
		        subDirs = System.IO.Directory.GetDirectories(currentDir);
		    }
		    // An UnauthorizedAccessException exception will be thrown if we do not have
		    // discovery permission on a folder or file. It may or may not be acceptable 
		    // to ignore the exception and continue enumerating the remaining files and 
		    // folders. It is also possible (but unlikely) that a DirectoryNotFound exception 
		    // will be raised. This will happen if currentDir has been deleted by
		    // another application or thread after our call to Directory.Exists. The 
		    // choice of which exceptions to catch depends entirely on the specific task 
		    // you are intending to perform and also on how much you know with certainty 
		    // about the systems on which this code will run.
		    catch (UnauthorizedAccessException e)
		    {                    
		        Console.WriteLine(e.Message);
		        continue;
		    }
		    catch (System.IO.DirectoryNotFoundException e)
		    {
		        Console.WriteLine(e.Message);
		        continue;
		    }
		
		    string[] slnfiles = null;
			string[] csprojfiles = null;
			string[] vsssccfiles = null;
		    string[] vspsccfiles = null;
		    try
		    {
		        slnfiles = System.IO.Directory.GetFiles(currentDir,"*.sln");
				csprojfiles = System.IO.Directory.GetFiles(currentDir,"*.csproj");
				vsssccfiles = System.IO.Directory.GetFiles(currentDir,"*.vssscc");
				vspsccfiles = System.IO.Directory.GetFiles(currentDir,"*.vspscc");
		    }
		
		    catch (UnauthorizedAccessException e)
		    {
		
		        Console.WriteLine(e.Message);
		        continue;
		    }
		
		    catch (System.IO.DirectoryNotFoundException e)
		    {
		        Console.WriteLine(e.Message);
		        continue;
		    }
		    // Perform the required action on each file here.
		    // Modify this block to perform your required task.
				
				#region SLN
			    foreach (string file in slnfiles)
			    {
			        try
			        {
			            // Perform whatever action is required in your scenario.
			            //System.IO.FileInfo fi = new System.IO.FileInfo(file);
			            //Console.WriteLine("{0}: {1}, {2}", fi.Name, fi.Length, fi.CreationTime);
						StripSLN(file);
			        }
			        catch (System.IO.FileNotFoundException e)
			        {
			            // If file was deleted by a separate application
			            //  or thread since the call to TraverseTree()
			            // then just continue.
			            Console.WriteLine(e.Message);
			            continue;
			        }
			    }
				#endregion
				
				#region CSPROJ
			    foreach (string file in csprojfiles)
			    {
			        try
			        {
			            // Perform whatever action is required in your scenario.
						StripCSPROJ(file);
			        }
			        catch (System.IO.FileNotFoundException e)
			        {
			            // If file was deleted by a separate application
			            //  or thread since the call to TraverseTree()
			            // then just continue.
			            Console.WriteLine(e.Message);
			            continue;
			        }
			    }
				#endregion
				#region VSSSCC
			    foreach (string file in vsssccfiles)
			    {
			        try
			        {
			            // Perform whatever action is required in your scenario.
			            Console.WriteLine("Removing file {0}", file);
						
						File.Delete(file);
			        }
			        catch (System.IO.FileNotFoundException e)
			        {
			            // If file was deleted by a separate application
			            //  or thread since the call to TraverseTree()
			            // then just continue.
			            Console.WriteLine(e.Message);
			            continue;
			        }
			    }
				#endregion

				#region VSPSCC
			    foreach (string file in vspsccfiles)
			    {
			        try
			        {
			            // Perform whatever action is required in your scenario.
			            Console.WriteLine("Removing file {0}", file);
						
						File.Delete(file);
			        }
			        catch (System.IO.FileNotFoundException e)
			        {
			            // If file was deleted by a separate application
			            //  or thread since the call to TraverseTree()
			            // then just continue.
			            Console.WriteLine(e.Message);
			            continue;
			        }
			    }
				#endregion

				
		    // Push the subdirectories onto the stack for traversal.
		    // This could also be done before handing the files.
		    foreach (string str in subDirs)
		        dirs.Push(str);
		}
		}	
	}
	#endregion
}
